(*
 *
 * Nombre: DateTimeConversion.pas
 * Fecha: Lunes 22 de Junio del 2009
 * Autor: Ivn Jurez Nez
 * eMail: radix@redmater.com
 * Pagina: www.redmater.com
 *
 * Unidad de codigo que muestra como hacer conversiones de tiempo para las distintas
 * zonas horarias del mundo en base a el formato UTC o Greenwich Mean Time usando la
 * especificacion RFC822 que puede ser encontrada en http://www.w3.org/Protocols/rfc822/#z28
 *
 * Este codigo se ofrece sin ninguna garantia y puede ser usado/modificado/redistribuido
 * con la unica condicion de incluir al autor en sus creditos.
 *)

unit DateTimeConversion;

interface
  uses SysUtils, StrUtils, DateUtils;

  FUNCTION ConvertZone(Zone:string):Integer;
  FUNCTION RFC822ToDateTime(val:string):TDateTime;
  FUNCTION DateToSpanish(date:TDateTime):string;

CONST
 ZoneCodes: Array[0..9] of Record
 Name: String;
 Time: ShortInt;
 end =
 ((Name:'UT' ; Time:0),
 (Name:'GMT' ; Time:0),
 (Name:'EST' ; Time:-5),
 (Name:'EDT' ; Time:-4),
 (Name:'CST' ; Time:-6),
 (Name:'CDT' ; Time:-5),
 (Name:'MST' ; Time:-7),
 (Name:'MDT' ; Time:-6),
 (Name:'PST' ; Time:-8),
 (Name:'PDT' ; Time:-7));

 implementation
uses
 MiscDateTime;

//Convierte una Zona por numero o texto: PDT, UT, GMT o +0000,+0600, etc a minutos
FUNCTION ConvertZone(Zone:string):Integer;
var
i: Byte;
ZoneConv: Integer;
BEGIN
ZoneConv := 0;

try
Zone := AnsiUpperCase(Trim(Zone));

 if (Zone[1]='+')OR(Zone[1]='-') then
 begin
   ZoneConv := (StrToInt(Zone[2]+Zone[3])*60)+StrToInt(Zone[4]+Zone[5]);
   if (Zone[1]='-') then
      ZoneConv := ZoneConv * -1;
 end;
 for i := 0 to Length(ZoneCodes) - 1 do
  if ZoneCodes[i].Name = Zone then
    ZoneConv := ZoneCodes[i].Time*60;
except
On Exception: EConvertError do
 ZoneConv := 0;
end;

result := ZoneConv;
END;
//Convierte el formato de Hora y Fecha RFC822 a TDateTime
FUNCTION RFC822ToDateTime(val:string):TDateTime;
var
 idx: byte;
 DateTimeC: TDateTime;
 Day,Year,Month,Time,Zone: string;
const
 Days: array[0..6] of string =
 ('SUN, ','MON, ','TUE, ','WED, ','THU, ','FRI, ','SAT, ');
 Months: array[0..11] of string =
 ('JAN','FEB','MAR','APR','MAY','JUN','JUL','AUG','SEP','OCT','NOV','DEC');
BEGIN

val := AnsiUpperCase(val);

for idx := 0 to Length(Days) - 1 do
  val := AnsiReplaceStr(val,Days[idx],'');

  {Podriamos usar un array y un for pero es preferible
  dejarlo asi para hacer el codigo mas mantenible}

  Day := Trim(Copy(val, 1, AnsiPos(' ', val)));
  Delete(val,1,Length(Day)+1);

  Month := Trim(Copy(val, 1, AnsiPos(' ', val)));
  Delete(val,1,Length(Month)+1);

for idx := 0 to Length(Months) - 1 do
  Month := AnsiReplaceStr(Month,Months[idx],IntToStr(idx+1));
{Si la ao esta declarado en 2 digitos lo pasamos a 4}
  Year := Trim(Copy(val, 1,AnsiPos(' ',val)));
  Delete(val,1,Length(Year)+1);
  if (StrToInt(Year)<1000) then
    Year := IntToStr(StrToInt(Year)+2000);

  Time := Trim(Copy(val, 1,AnsiPos(' ',val)));
  Delete(val,1,Length(Time)+1);
  DateTimeC := StrToDateTime(Day+'/'+Month+'/'+Year+' '+Time);

  Zone := Trim(Copy(val,1,Length(val)));
{Convertimos la fecha RFC9822 al cambio horario nuestro}
  DateTimeC := IncSecond(DateTimeC,GetTimeZone*3600);

result := DateTimeC;
END;
//Convierte una Fecha TDateTime con formato DD/MM/AAAA a formato Espaol
//como: Lunes, 23 de Agosto del 2009
FUNCTION DateToSpanish(date:TDateTime):string;
VAR
Year, Month, Day: word;
CONST
Dias: Array[1..7] of String =
('Domingo','Lunes','Martes','Miercoles','Jueves','Viernes','Sabado');
Meses: Array[1..12] of String =
('Enero','Febero','Marzo','Abril','Mayo','Junio','Julio','Agosto','Septiembre',
'Octubre','Noviembre','Diciembre');
BEGIN
DecodeDate(date,Year,Month,Day);

  Result := Dias[DayOfWeek(date)]+', '+InttoStr(Day)+' de '+Meses[Month]+' de '+ IntToStr(Year);
END;

end.
